@@ -121,14 +121,14 @@
 #line 25 "idmef-criteria-string.yac.y"
 
 #define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_IDMEF_CRITERIA
-        
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <stdarg.h>
 #include <ctype.h>
-        
+
 #include "prelude-thread.h"
 #include "prelude-log.h"
 #include "prelude-error.h"
@@ -136,16 +136,17 @@
 
 #include "idmef.h"
 #include "idmef-criteria.h"
+#include "common.h"
 
 
 static int real_ret = 0;
 static idmef_criteria_t *processed_criteria;
 pthread_mutex_t _criteria_parse_mutex = PTHREAD_MUTEX_INITIALIZER;
 
- 
+
 #define operator_or 1
 #define operator_and 2
- 
+
 extern int yylex(void);
 extern void yylex_init(void);
 extern void yylex_destroy(void);
@@ -178,16 +179,16 @@ void _idmef_criteria_string_init_lexer(void);
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 64 "idmef-criteria-string.yac.y"
+#line 65 "idmef-criteria-string.yac.y"
 {
-	char *str;
+        char *str;
         int operator;
-	idmef_criterion_t *criterion;
-	idmef_criteria_t *criteria;
-	idmef_criterion_operator_t relation;
+        idmef_criterion_t *criterion;
+        idmef_criteria_t *criteria;
+        idmef_criterion_operator_t relation;
 }
 /* Line 187 of yacc.c.  */
-#line 191 "idmef-criteria-string.yac.c"
+#line 192 "idmef-criteria-string.yac.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -200,7 +201,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 204 "idmef-criteria-string.yac.c"
+#line 205 "idmef-criteria-string.yac.c"
 
 #ifdef short
 # undef short
@@ -491,9 +492,9 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   116,   116,   121,   124,   135,   146,   151,   186,   209,
-     234,   235,   236,   237,   238,   239,   240,   241,   242,   243,
-     244,   245,   246,   247,   248,   249,   250,   251,   255,   256
+       0,   117,   117,   122,   125,   136,   147,   152,   187,   210,
+     235,   236,   237,   238,   239,   240,   241,   242,   243,   244,
+     245,   246,   247,   248,   249,   250,   251,   252,   256,   257
 };
 #endif
 
@@ -1117,19 +1118,19 @@ yydestruct (yymsg, yytype, yyvaluep)
   switch (yytype)
     {
       case 3: /* "TOK_IDMEF_VALUE" */
-#line 77 "idmef-criteria-string.yac.y"
+#line 78 "idmef-criteria-string.yac.y"
 	{ free ((yyvaluep->str)); };
-#line 1123 "idmef-criteria-string.yac.c"
+#line 1124 "idmef-criteria-string.yac.c"
 	break;
       case 4: /* "TOK_IDMEF_PATH" */
-#line 77 "idmef-criteria-string.yac.y"
+#line 78 "idmef-criteria-string.yac.y"
 	{ free ((yyvaluep->str)); };
-#line 1128 "idmef-criteria-string.yac.c"
+#line 1129 "idmef-criteria-string.yac.c"
 	break;
       case 29: /* "criteria" */
-#line 78 "idmef-criteria-string.yac.y"
+#line 79 "idmef-criteria-string.yac.y"
 	{ idmef_criteria_destroy((yyvaluep->criteria)); };
-#line 1133 "idmef-criteria-string.yac.c"
+#line 1134 "idmef-criteria-string.yac.c"
 	break;
 
       default:
@@ -1438,250 +1439,250 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 116 "idmef-criteria-string.yac.y"
+#line 117 "idmef-criteria-string.yac.y"
     {
-							processed_criteria = (yyvsp[(1) - (1)].criteria);
-						}
+                                                        processed_criteria = (yyvsp[(1) - (1)].criteria);
+                                                }
     break;
 
   case 3:
-#line 121 "idmef-criteria-string.yac.y"
+#line 122 "idmef-criteria-string.yac.y"
     {
-							(yyval.criteria) = (yyvsp[(1) - (1)].criteria);
-						}
+                                                        (yyval.criteria) = (yyvsp[(1) - (1)].criteria);
+                                                }
     break;
 
   case 4:
-#line 124 "idmef-criteria-string.yac.y"
-    {                                                        
+#line 125 "idmef-criteria-string.yac.y"
+    {
                                                         if ( (yyvsp[(2) - (3)].operator) == operator_or ) {
                                                                 idmef_criteria_or_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
                                                         } else {
                                                                 idmef_criteria_and_criteria((yyvsp[(1) - (3)].criteria), (yyvsp[(3) - (3)].criteria));
                                                         }
-                                                        
+
                                                         (yyval.criteria) = (yyvsp[(1) - (3)].criteria);
-						}
+                                                }
     break;
 
   case 5:
-#line 135 "idmef-criteria-string.yac.y"
+#line 136 "idmef-criteria-string.yac.y"
     {
-							idmef_criteria_t *criteria;
+                                                        idmef_criteria_t *criteria;
 
-							real_ret = idmef_criteria_new(&criteria);
-							if ( real_ret < 0 )
-								YYABORT;
+                                                        real_ret = idmef_criteria_new(&criteria);
+                                                        if ( real_ret < 0 )
+                                                                YYABORT;
 
                                                         idmef_criteria_set_criterion(criteria, (yyvsp[(1) - (1)].criterion));
-							(yyval.criteria) = criteria;
-                                                        
-						}
+                                                        (yyval.criteria) = criteria;
+
+                                                }
     break;
 
   case 6:
-#line 146 "idmef-criteria-string.yac.y"
+#line 147 "idmef-criteria-string.yac.y"
     {
-							(yyval.criteria) = (yyvsp[(2) - (3)].criteria);
-						}
+                                                        (yyval.criteria) = (yyvsp[(2) - (3)].criteria);
+                                                }
     break;
 
   case 7:
-#line 151 "idmef-criteria-string.yac.y"
+#line 152 "idmef-criteria-string.yac.y"
     {
-							idmef_path_t *path = NULL;
-							idmef_criterion_value_t *value = NULL;
-							idmef_criterion_operator_t operator = (yyvsp[(2) - (3)].relation);
-							idmef_criterion_t *criterion;
+                                                        idmef_path_t *path = NULL;
+                                                        idmef_criterion_value_t *value = NULL;
+                                                        idmef_criterion_operator_t operator = (yyvsp[(2) - (3)].relation);
+                                                        idmef_criterion_t *criterion;
 
-							real_ret = idmef_path_new_fast(&path, (yyvsp[(1) - (3)].str));
-							if ( real_ret < 0 ) {
-								free((yyvsp[(1) - (3)].str));
-								free((yyvsp[(3) - (3)].str));
-								YYABORT;
-							}
-                                                        
-							real_ret = idmef_criterion_value_new_from_string(&value, path, (yyvsp[(3) - (3)].str), operator);
-							if ( real_ret < 0 ) {
-								free((yyvsp[(1) - (3)].str));
-								free((yyvsp[(3) - (3)].str));
-								idmef_path_destroy(path);
-								YYABORT;
-							}
+                                                        real_ret = idmef_path_new_fast(&path, (yyvsp[(1) - (3)].str));
+                                                        if ( real_ret < 0 ) {
+                                                                free((yyvsp[(1) - (3)].str));
+                                                                free((yyvsp[(3) - (3)].str));
+                                                                YYABORT;
+                                                        }
 
-							real_ret = idmef_criterion_new(&criterion, path, value, operator);
-							if ( real_ret < 0 ) {
-								free((yyvsp[(1) - (3)].str));
-								free((yyvsp[(3) - (3)].str));
-								idmef_path_destroy(path);
-								idmef_criterion_value_destroy(value);
-								YYABORT;
-							}
+                                                        real_ret = idmef_criterion_value_new_from_string(&value, path, (yyvsp[(3) - (3)].str), operator);
+                                                        if ( real_ret < 0 ) {
+                                                                free((yyvsp[(1) - (3)].str));
+                                                                free((yyvsp[(3) - (3)].str));
+                                                                idmef_path_destroy(path);
+                                                                YYABORT;
+                                                        }
 
-							free((yyvsp[(1) - (3)].str));
-							free((yyvsp[(3) - (3)].str));
-                                                        
-							(yyval.criterion) = criterion;
-						}
+                                                        real_ret = idmef_criterion_new(&criterion, path, value, operator);
+                                                        if ( real_ret < 0 ) {
+                                                                free((yyvsp[(1) - (3)].str));
+                                                                free((yyvsp[(3) - (3)].str));
+                                                                idmef_path_destroy(path);
+                                                                idmef_criterion_value_destroy(value);
+                                                                YYABORT;
+                                                        }
+
+                                                        free((yyvsp[(1) - (3)].str));
+                                                        free((yyvsp[(3) - (3)].str));
+
+                                                        (yyval.criterion) = criterion;
+                                                }
     break;
 
   case 8:
-#line 186 "idmef-criteria-string.yac.y"
+#line 187 "idmef-criteria-string.yac.y"
     {
-							idmef_path_t *path;
-							idmef_criterion_t *criterion;
-                                                        
-							real_ret = idmef_path_new_fast(&path, (yyvsp[(1) - (1)].str));
-							if ( real_ret < 0 ) {
-								free((yyvsp[(1) - (1)].str));
-								YYABORT;
-							}
+                                                        idmef_path_t *path;
+                                                        idmef_criterion_t *criterion;
 
-							real_ret = idmef_criterion_new(&criterion, path, NULL,
+                                                        real_ret = idmef_path_new_fast(&path, (yyvsp[(1) - (1)].str));
+                                                        if ( real_ret < 0 ) {
+                                                                free((yyvsp[(1) - (1)].str));
+                                                                YYABORT;
+                                                        }
+
+                                                        real_ret = idmef_criterion_new(&criterion, path, NULL,
                                                                                        IDMEF_CRITERION_OPERATOR_NOT|
                                                                                        IDMEF_CRITERION_OPERATOR_NULL);
-							if ( real_ret < 0 ) {
-								free((yyvsp[(1) - (1)].str));
-								idmef_path_destroy(path);
-								YYABORT;
-							}
+                                                        if ( real_ret < 0 ) {
+                                                                free((yyvsp[(1) - (1)].str));
+                                                                idmef_path_destroy(path);
+                                                                YYABORT;
+                                                        }
 
-							free((yyvsp[(1) - (1)].str));
+                                                        free((yyvsp[(1) - (1)].str));
 
-							(yyval.criterion) = criterion;
-						}
+                                                        (yyval.criterion) = criterion;
+                                                }
     break;
 
   case 9:
-#line 209 "idmef-criteria-string.yac.y"
+#line 210 "idmef-criteria-string.yac.y"
     {
-							idmef_path_t *path;
-							idmef_criterion_t *criterion;
+                                                        idmef_path_t *path;
+                                                        idmef_criterion_t *criterion;
 
-							real_ret = idmef_path_new_fast(&path, (yyvsp[(2) - (2)].str));
-							if ( real_ret < 0) {
-								free((yyvsp[(2) - (2)].str));
-								YYABORT;
-							}
+                                                        real_ret = idmef_path_new_fast(&path, (yyvsp[(2) - (2)].str));
+                                                        if ( real_ret < 0) {
+                                                                free((yyvsp[(2) - (2)].str));
+                                                                YYABORT;
+                                                        }
 
-							real_ret = idmef_criterion_new(&criterion, path, NULL,
+                                                        real_ret = idmef_criterion_new(&criterion, path, NULL,
                                                                                        IDMEF_CRITERION_OPERATOR_NULL);
-							if ( real_ret < 0 ) {
-								free((yyvsp[(2) - (2)].str));
-								idmef_path_destroy(path);
-								YYABORT;
-							}
+                                                        if ( real_ret < 0 ) {
+                                                                free((yyvsp[(2) - (2)].str));
+                                                                idmef_path_destroy(path);
+                                                                YYABORT;
+                                                        }
 
-							free((yyvsp[(2) - (2)].str));
+                                                        free((yyvsp[(2) - (2)].str));
 
-							(yyval.criterion) = criterion;
-						}
+                                                        (yyval.criterion) = criterion;
+                                                }
     break;
 
   case 10:
-#line 234 "idmef-criteria-string.yac.y"
+#line 235 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR; }
     break;
 
   case 11:
-#line 235 "idmef-criteria-string.yac.y"
+#line 236 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 12:
-#line 236 "idmef-criteria-string.yac.y"
+#line 237 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOT; }
     break;
 
   case 13:
-#line 237 "idmef-criteria-string.yac.y"
+#line 238 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_SUBSTR|IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 14:
-#line 238 "idmef-criteria-string.yac.y"
+#line 239 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX; }
     break;
 
   case 15:
-#line 239 "idmef-criteria-string.yac.y"
+#line 240 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 16:
-#line 240 "idmef-criteria-string.yac.y"
+#line 241 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOT; }
     break;
 
   case 17:
-#line 241 "idmef-criteria-string.yac.y"
+#line 242 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_REGEX|IDMEF_CRITERION_OPERATOR_NOT|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 18:
-#line 242 "idmef-criteria-string.yac.y"
+#line 243 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_GREATER; }
     break;
 
   case 19:
-#line 243 "idmef-criteria-string.yac.y"
+#line 244 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_GREATER|IDMEF_CRITERION_OPERATOR_EQUAL; }
     break;
 
   case 20:
-#line 244 "idmef-criteria-string.yac.y"
+#line 245 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_LESSER; }
     break;
 
   case 21:
-#line 245 "idmef-criteria-string.yac.y"
+#line 246 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_EQUAL; }
     break;
 
   case 22:
-#line 246 "idmef-criteria-string.yac.y"
+#line 247 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL; }
     break;
 
   case 23:
-#line 247 "idmef-criteria-string.yac.y"
+#line 248 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE; }
     break;
 
   case 24:
-#line 248 "idmef-criteria-string.yac.y"
+#line 249 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOT; }
     break;
 
   case 25:
-#line 249 "idmef-criteria-string.yac.y"
+#line 250 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE|IDMEF_CRITERION_OPERATOR_NOT; }
     break;
 
   case 26:
-#line 250 "idmef-criteria-string.yac.y"
+#line 251 "idmef-criteria-string.yac.y"
     { (yyval.relation) = IDMEF_CRITERION_OPERATOR_NULL; }
     break;
 
   case 27:
-#line 251 "idmef-criteria-string.yac.y"
-    { real_ret = prelude_error_verbose(PRELUDE_ERROR_IDMEF_CRITERIA_PARSE, 
+#line 252 "idmef-criteria-string.yac.y"
+    { real_ret = prelude_error_verbose(PRELUDE_ERROR_IDMEF_CRITERIA_PARSE,
                                                                        "Criteria parser reported: Invalid operator found"); YYERROR; }
     break;
 
   case 28:
-#line 255 "idmef-criteria-string.yac.y"
+#line 256 "idmef-criteria-string.yac.y"
     { (yyval.operator) = operator_and; }
     break;
 
   case 29:
-#line 256 "idmef-criteria-string.yac.y"
+#line 257 "idmef-criteria-string.yac.y"
     { (yyval.operator) = operator_or; }
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1685 "idmef-criteria-string.yac.c"
+#line 1686 "idmef-criteria-string.yac.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1895,7 +1896,7 @@ yyreturn:
 }
 
 
-#line 259 "idmef-criteria-string.yac.y"
+#line 260 "idmef-criteria-string.yac.y"
 
 
 
@@ -1910,33 +1911,35 @@ static void yyerror(char *s)  /* Called by yyparse on error */
 int idmef_criteria_new_from_string(idmef_criteria_t **new_criteria, const char *str)
 {
         int ret;
-	void *state;
-        
-	prelude_thread_mutex_lock(&_criteria_parse_mutex);
+        void *state;
+
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        prelude_thread_mutex_lock(&_criteria_parse_mutex);
 
         real_ret = 0;
-	processed_criteria = NULL;
-        
-	state = yy_scan_string(str);
-	ret = yyparse();
-	yy_delete_buffer(state);
+        processed_criteria = NULL;
+
+        state = yy_scan_string(str);
+        ret = yyparse();
+        yy_delete_buffer(state);
 
         if ( ret != 0 ) {
                 _idmef_criteria_string_init_lexer();
-                
+
                 if ( real_ret )
                         ret = real_ret;
                 else
                         ret = prelude_error_make(PRELUDE_ERROR_SOURCE_IDMEF_CRITERIA, PRELUDE_ERROR_IDMEF_CRITERIA_PARSE);
-	
-	        if ( processed_criteria )
-	                idmef_criteria_destroy(processed_criteria);
-	}
+
+                if ( processed_criteria )
+                        idmef_criteria_destroy(processed_criteria);
+        }
 
         else *new_criteria = processed_criteria;
 
-	prelude_thread_mutex_unlock(&_criteria_parse_mutex);
+        prelude_thread_mutex_unlock(&_criteria_parse_mutex);
 
-	return ret;
+        return ret;
 }
 
