@@ -44,8 +44,6 @@ struct idmef_criterion {
 
 
 struct idmef_criteria {
-        prelude_list_t list;
-
         idmef_criterion_t *criterion;
         struct idmef_criteria *or;
         struct idmef_criteria *and;
@@ -118,8 +116,8 @@ const char *idmef_criterion_operator_to_string(idmef_criterion_operator_t op)
 int idmef_criterion_new(idmef_criterion_t **criterion, idmef_path_t *path,
                         idmef_criterion_value_t *value, idmef_criterion_operator_t op)
 {
-        prelude_return_val_if_fail(path != NULL, -1);
-        prelude_return_val_if_fail(! (value == NULL && ! (op & IDMEF_CRITERION_OPERATOR_NULL)), -1);
+        prelude_return_val_if_fail(path != NULL, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(! (value == NULL && ! (op & IDMEF_CRITERION_OPERATOR_NULL)), prelude_error(PRELUDE_ERROR_ASSERTION));
 
         *criterion = calloc(1, sizeof(**criterion));
         if ( ! *criterion )
@@ -169,7 +167,7 @@ int idmef_criterion_clone(const idmef_criterion_t *criterion, idmef_criterion_t
         idmef_path_t *path;
         idmef_criterion_value_t *value = NULL;
 
-        prelude_return_val_if_fail(criterion, -1);
+        prelude_return_val_if_fail(criterion, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         ret = idmef_path_clone(criterion->path, &path);
         if ( ret < 0 )
@@ -213,7 +211,8 @@ int idmef_criterion_print(const idmef_criterion_t *criterion, prelude_io_t *fd)
         int ret;
         prelude_string_t *out;
 
-        prelude_return_val_if_fail(criterion && fd, -1);
+        prelude_return_val_if_fail(criterion, prelude_error(PRELUDE_ERROR_ASSERTION));
+	prelude_return_val_if_fail(fd, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         ret = prelude_string_new(&out);
         if ( ret < 0 )
@@ -250,7 +249,8 @@ int idmef_criterion_to_string(const idmef_criterion_t *criterion, prelude_string
 {
         const char *name, *operator;
 
-        prelude_return_val_if_fail(criterion && out, -1);
+        prelude_return_val_if_fail(criterion, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(out, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         operator = idmef_criterion_operator_to_string(criterion->operator);
         if ( ! operator )
@@ -313,7 +313,7 @@ idmef_criterion_value_t *idmef_criterion_get_value(const idmef_criterion_t *crit
  */
 idmef_criterion_operator_t idmef_criterion_get_operator(const idmef_criterion_t *criterion)
 {
-        prelude_return_val_if_fail(criterion, -1);
+        prelude_return_val_if_fail(criterion, prelude_error(PRELUDE_ERROR_ASSERTION));
         return criterion->operator;
 }
 
@@ -335,7 +335,9 @@ int idmef_criterion_match(const idmef_criterion_t *criterion, idmef_message_t *m
         int ret;
         idmef_value_t *value;
 
-        prelude_return_val_if_fail(criterion && message, -1);
+        prelude_return_val_if_fail(criterion, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(message, prelude_error(PRELUDE_ERROR_ASSERTION));
+
 
         ret = idmef_path_get(criterion->path, message, &value);
         if ( ret < 0 )
@@ -424,13 +426,13 @@ int idmef_criteria_clone(idmef_criteria_t *src, idmef_criteria_t **dst)
         int ret;
         idmef_criteria_t *new;
 
-        prelude_return_val_if_fail(src, -1);
+        prelude_return_val_if_fail(src, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         new = *dst = malloc(sizeof(*new));
         if ( ! new )
                 return prelude_error_from_errno(errno);
 
-        memcpy(new, src, sizeof(*new));
+        new->or = new->and = NULL;
 
         if ( src->or ) {
                 ret = idmef_criteria_clone(src->or, &new->or);
@@ -480,7 +482,8 @@ int idmef_criteria_print(const idmef_criteria_t *criteria, prelude_io_t *fd)
         int ret;
         prelude_string_t *out;
 
-        prelude_return_val_if_fail(criteria && fd, -1);
+        prelude_return_val_if_fail(criteria, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(fd, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         ret = prelude_string_new(&out);
         if ( ret < 0 )
@@ -500,7 +503,8 @@ int idmef_criteria_print(const idmef_criteria_t *criteria, prelude_io_t *fd)
 
 int idmef_criteria_to_string(const idmef_criteria_t *criteria, prelude_string_t *out)
 {
-        prelude_return_val_if_fail(criteria && out, -1);
+        prelude_return_val_if_fail(criteria, prelude_error(PRELUDE_ERROR_ASSERTION));
+	prelude_return_val_if_fail(out, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         if ( criteria->or )
                 prelude_string_sprintf(out, "((");
@@ -541,16 +545,13 @@ idmef_criterion_t *idmef_criteria_get_criterion(const idmef_criteria_t *criteria
 
 void idmef_criteria_or_criteria(idmef_criteria_t *criteria, idmef_criteria_t *criteria2)
 {
-        idmef_criteria_t *last = NULL;
+        prelude_return_if_fail(criteria);
+	prelude_return_if_fail(criteria2);
 
-        prelude_return_if_fail(criteria && criteria2);
-
-        while ( criteria ) {
-                last = criteria;
+        while ( criteria->or )
                 criteria = criteria->or;
-        }
 
-        last->or = criteria2;
+        criteria->or = criteria2;
 }
 
 
@@ -560,7 +561,8 @@ int idmef_criteria_and_criteria(idmef_criteria_t *criteria, idmef_criteria_t *cr
         int ret;
         idmef_criteria_t *new, *last = NULL;
 
-        prelude_return_val_if_fail(criteria && criteria2, -1);
+        prelude_return_val_if_fail(criteria, prelude_error(PRELUDE_ERROR_ASSERTION)); 
+	prelude_return_val_if_fail(criteria2, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         while ( criteria ) {
                 last = criteria;
@@ -588,7 +590,9 @@ int idmef_criteria_and_criteria(idmef_criteria_t *criteria, idmef_criteria_t *cr
 
 void idmef_criteria_set_criterion(idmef_criteria_t *criteria, idmef_criterion_t *criterion)
 {
-        prelude_return_if_fail(criteria && criterion);
+        prelude_return_if_fail(criteria);
+	prelude_return_if_fail(criterion);
+
         criteria->criterion = criterion;
 }
 
@@ -609,7 +613,8 @@ int idmef_criteria_match(const idmef_criteria_t *criteria, idmef_message_t *mess
         int ret;
         idmef_criteria_t *next;
 
-        prelude_return_val_if_fail(criteria && message, -1);
+        prelude_return_val_if_fail(criteria, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(message, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         ret = idmef_criterion_match(criteria->criterion, message);
         if ( ret < 0 )
