@@ -44,7 +44,7 @@
                         IDMEF_CRITERION_OPERATOR_EQUAL
 
 #define DATA_OPERATOR    IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOT| \
-                         IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_GREATER
+                         IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_GREATER|IDMEF_CRITERION_OPERATOR_SUBSTR
 
 #define TIME_OPERATOR    IDMEF_CRITERION_OPERATOR_LESSER|IDMEF_CRITERION_OPERATOR_GREATER| \
                          IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOT
@@ -231,7 +231,9 @@ static int enum_write(const idmef_value_type_t *src, prelude_string_t *out)
         const char *str;
 
         str = idmef_class_enum_to_string(src->data.enum_val.class_id, src->data.enum_val.value);
-
+        if ( ! str ) 
+                return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, "Enumeration conversion from numeric to string failed");
+                
         return prelude_string_cat(out, str);
 }
 
@@ -382,6 +384,23 @@ static int data_compare(const idmef_value_type_t *t1, const idmef_value_type_t *
                         size_t len, idmef_criterion_operator_t op)
 {
         int ret;
+        size_t s1_len, s2_len;
+        const void *s1 = NULL, *s2 = NULL;
+
+        if ( t1->data.data_val )
+                s1 = idmef_data_get_data(t1->data.data_val);
+
+        if ( t2->data.string_val )
+                s2 = idmef_data_get_data(t2->data.data_val);
+        
+        if ( ! s1 || ! s2 )
+                return (s1) ? 1 : -1;                
+        
+        if ( op & IDMEF_CRITERION_OPERATOR_SUBSTR ) {
+                s1_len = idmef_data_get_len(t1->data.data_val);
+                s2_len = idmef_data_get_len(t2->data.data_val);
+                return ( memmem(s1, s1_len, s2, s2_len) ) ? 0 : -1;
+        }
         
         ret = idmef_data_compare(t1->data.data_val, t2->data.data_val);        
         if ( ret == 0 && op & IDMEF_CRITERION_OPERATOR_EQUAL )
@@ -392,7 +411,8 @@ static int data_compare(const idmef_value_type_t *t1, const idmef_value_type_t *
 
         else if ( ret > 0 && op & IDMEF_CRITERION_OPERATOR_GREATER )
                 return 0;
-                                
+        
+                       
         return -1;
 }
 
