@@ -1029,7 +1029,7 @@ int prelude_client_new(prelude_client_t **client, const char *profile)
         int ret;
         prelude_client_t *new;
 
-        prelude_return_val_if_fail(profile, -1);
+        prelude_return_val_if_fail(profile, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         new = calloc(1, sizeof(*new));
         if ( ! new )
@@ -1110,7 +1110,7 @@ int prelude_client_init(prelude_client_t *client)
         prelude_string_t *err;
         prelude_option_warning_t old_warnings;
 
-        prelude_return_val_if_fail(client, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         prelude_option_set_warnings(0, &old_warnings);
 
@@ -1157,7 +1157,7 @@ int prelude_client_start(prelude_client_t *client)
         int ret;
         void *credentials;
 
-        prelude_return_val_if_fail(client, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         if ( client->status == CLIENT_STATUS_NEED_INIT ) {
                 /*
@@ -1232,7 +1232,8 @@ idmef_analyzer_t *prelude_client_get_analyzer(prelude_client_t *client)
  */
 void prelude_client_send_msg(prelude_client_t *client, prelude_msg_t *msg)
 {
-        prelude_return_if_fail(client && msg);
+        prelude_return_if_fail(client);
+        prelude_return_if_fail(msg);
 
         if ( client->flags & PRELUDE_CLIENT_FLAGS_ASYNC_SEND )
                 prelude_connection_pool_broadcast_async(client->cpool, msg);
@@ -1254,7 +1255,8 @@ void prelude_client_send_msg(prelude_client_t *client, prelude_msg_t *msg)
  */
 void prelude_client_send_idmef(prelude_client_t *client, idmef_message_t *msg)
 {
-        prelude_return_if_fail(client && msg);
+        prelude_return_if_fail(client);
+        prelude_return_if_fail(msg);
 
         /*
          * we need to hold a lock since asynchronous heartbeat
@@ -1299,7 +1301,9 @@ prelude_connection_pool_t *prelude_client_get_connection_pool(prelude_client_t *
  */
 void prelude_client_set_connection_pool(prelude_client_t *client, prelude_connection_pool_t *pool)
 {
-        prelude_return_if_fail(client && pool);
+        prelude_return_if_fail(client);
+        prelude_return_if_fail(pool);
+
         client->cpool = pool;
 }
 
@@ -1316,7 +1320,9 @@ void prelude_client_set_connection_pool(prelude_client_t *client, prelude_connec
 void prelude_client_set_heartbeat_cb(prelude_client_t *client,
                                      void (*cb)(prelude_client_t *client, idmef_message_t *hb))
 {
-        prelude_return_if_fail(client && cb);
+        prelude_return_if_fail(client);
+        prelude_return_if_fail(cb);
+
         client->heartbeat_cb = cb;
 }
 
@@ -1374,7 +1380,7 @@ int prelude_client_set_flags(prelude_client_t *client, prelude_client_flags_t fl
 {
         int ret = 0;
 
-        prelude_return_val_if_fail(client, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         client->flags = flags;
 
@@ -1404,7 +1410,7 @@ int prelude_client_set_flags(prelude_client_t *client, prelude_client_flags_t fl
  */
 prelude_client_flags_t prelude_client_get_flags(prelude_client_t *client)
 {
-        prelude_return_val_if_fail(client, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
         return client->flags;
 }
 
@@ -1418,7 +1424,7 @@ prelude_client_flags_t prelude_client_get_flags(prelude_client_t *client)
  */
 prelude_connection_permission_t prelude_client_get_required_permission(prelude_client_t *client)
 {
-        prelude_return_val_if_fail(client, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
         return client->permission;
 }
 
@@ -1480,7 +1486,8 @@ const char *prelude_client_get_config_filename(prelude_client_t *client)
  */
 int prelude_client_set_config_filename(prelude_client_t *client, const char *filename)
 {
-        prelude_return_val_if_fail(client && filename, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(filename, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         if ( client->config_filename )
                 free(client->config_filename);
@@ -1555,22 +1562,22 @@ const char *prelude_client_get_setup_error(prelude_client_t *client)
                 prelude_connection_permission_to_string(client->permission, perm);
 
                 ret = prelude_string_sprintf(out, "\nProfile '%s' does not exist. In order to create it, please run:\n"
-                                             "prelude-admin register %s \"%s\" <manager address> --uid %d --gid %d",
+                                             "prelude-admin register \"%s\" \"%s\" <manager address> --uid %d --gid %d",
                                              prelude_client_profile_get_name(client->profile),
                                              prelude_client_profile_get_name(client->profile),
                                              prelude_string_get_string(perm),
-                                             prelude_client_profile_get_uid(client->profile),
-                                             prelude_client_profile_get_gid(client->profile));
+                                             (int) prelude_client_profile_get_uid(client->profile),
+                                             (int) prelude_client_profile_get_gid(client->profile));
 
                 prelude_string_destroy(perm);
 
         } else {
                 ret = prelude_string_sprintf(out, "\nProfile '%s' does not exist. In order to create it, please run:\n"
-                                             "prelude-admin add %s --uid %d --gid %d",
+                                             "prelude-admin add \"%s\" --uid %d --gid %d",
                                              prelude_client_profile_get_name(client->profile),
                                              prelude_client_profile_get_name(client->profile),
-                                             prelude_client_profile_get_uid(client->profile),
-                                             prelude_client_profile_get_gid(client->profile));
+                                             (int) prelude_client_profile_get_uid(client->profile),
+                                             (int) prelude_client_profile_get_gid(client->profile));
         }
 
         if ( ret < 0 )
@@ -1597,7 +1604,9 @@ int prelude_client_handle_msg_default(prelude_client_t *client, prelude_msg_t *m
         int ret;
         uint8_t tag;
 
-        prelude_return_val_if_fail(client && msg && msgbuf, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(msg, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(msgbuf, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         tag = prelude_msg_get_tag(msg);
         if ( tag != PRELUDE_MSG_OPTION_REQUEST )
@@ -1622,7 +1631,7 @@ int prelude_client_new_msgbuf(prelude_client_t *client, prelude_msgbuf_t **msgbu
 {
         int ret;
 
-        prelude_return_val_if_fail(client, -1);
+        prelude_return_val_if_fail(client, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         ret = prelude_msgbuf_new(msgbuf);
         if ( ret < 0 )
