@@ -292,7 +292,7 @@ static int journal_initialize(prelude_failover_t *failover, const char *filename
         struct stat jst, wst;
         failover_journal_entry_t jentry;
 
-        failover->jfd = open(filename, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR);
+        failover->jfd = open(filename, O_CREAT|O_RDWR, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
         if ( failover->jfd < 0 )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not open '%s': %s", filename, strerror(errno));
 
@@ -330,12 +330,12 @@ static int journal_initialize(prelude_failover_t *failover, const char *filename
 }
 
 
-static int lock_cmd(int fd, int cmd, int type)
+static int lock_cmd(const char *filename, int fd, int cmd, int type)
 {
         int ret;
         struct flock lock;
 
-        lock.l_type = type;    /* write lock */
+        lock.l_type = type;       /* write lock */
         lock.l_start = 0;         /* from offset 0 */
         lock.l_whence = SEEK_SET; /* at the beginning of the file */
         lock.l_len = 0;           /* until EOF */
@@ -345,7 +345,8 @@ static int lock_cmd(int fd, int cmd, int type)
                 if ( errno == EAGAIN || errno == EACCES )
                         return 0;
 
-                return prelude_error_from_errno(errno);
+                return prelude_error_verbose(prelude_error_code_from_errno(errno), "error locking '%s': %s",
+                                             filename, strerror(errno));
         }
 
         return 1;
@@ -357,11 +358,11 @@ static int open_exclusive(const char *filename, int flags, int *fd)
 {
         int ret;
 
-        *fd = open(filename, flags, S_IRUSR|S_IWUSR);
+        *fd = open(filename, flags, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
         if ( *fd < 0 )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "error opening '%s': %s", filename, strerror(errno));
 
-        ret = lock_cmd(*fd, F_SETLK, F_RDLCK|F_WRLCK);
+        ret = lock_cmd(filename, *fd, F_SETLK, F_RDLCK|F_WRLCK);
         if ( ret <= 0 )
                 close(*fd);
 
@@ -498,33 +499,46 @@ error:
 
 int prelude_failover_new(prelude_failover_t **out, const char *dirname)
 {
+        mode_t mode;
         size_t flen;
         int ret, wfd, rfd;
         char filename[PATH_MAX];
         prelude_failover_t *new;
 
+        mode = umask(S_IRWXO);
+
         ret = mkdir(dirname, S_IRWXU|S_IRWXG);
-        if ( ret < 0 && errno != EEXIST )
+        if ( ret < 0 && errno != EEXIST ) {
+                umask(mode);
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not create directory '%s': %s", dirname, strerror(errno));
+        }
 
         wfd = get_failover_data_filename_and_fd(dirname, filename, sizeof(filename));
-        if ( wfd < 0 )
+        if ( wfd < 0 ) {
+                umask(mode);
                 return wfd;
+        }
 
         rfd = open(filename, O_RDONLY);
         if ( rfd < 0 ) {
+                umask(mode);
                 close(wfd);
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "could not open '%s' for reading: %s", filename, strerror(errno));
         }
 
         new = calloc(1, sizeof(*new));
-        if ( ! new )
+        if ( ! new ) {
+                umask(mode);
+                close(rfd);
+                close(wfd);
                 return prelude_error_from_errno(errno);
+        }
 
         new->jfd = -1;
 
         ret = prelude_io_new(&new->wfd);
         if ( ret < 0 ) {
+                umask(mode);
                 close(rfd);
                 close(wfd);
                 free(new);
@@ -533,6 +547,7 @@ int prelude_failover_new(prelude_failover_t **out, const char *dirname)
 
         ret = prelude_io_new(&new->rfd);
         if ( ret < 0 ) {
+                umask(mode);
                 close(rfd);
                 close(wfd);
                 free(new);
@@ -546,16 +561,19 @@ int prelude_failover_new(prelude_failover_t **out, const char *dirname)
 
         ret = snprintf(filename + flen, sizeof(filename) - flen, ".journal");
         if ( ret < 0 || ret >= (sizeof(filename) - flen) ) {
+                umask(mode);
                 prelude_failover_destroy(new);
                 return -1;
         }
 
         ret = journal_initialize(new, filename);
         if ( ret < 0 ) {
+                umask(mode);
                 prelude_failover_destroy(new);
                 return ret;
         }
 
+        umask(mode);
         *out = new;
 
         return 0;
