@@ -218,7 +218,7 @@ int prelude_string_new_dup_fast(prelude_string_t **string, const char *str, size
 {
         int ret;
 
-        prelude_return_val_if_fail(str, -1);
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
         STRING_RETURN_IF_INVALID(str, len);
 
         ret = prelude_string_new(string);
@@ -250,7 +250,7 @@ int prelude_string_new_dup_fast(prelude_string_t **string, const char *str, size
  */
 int prelude_string_new_dup(prelude_string_t **string, const char *str)
 {
-        prelude_return_val_if_fail(str, -1);
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
         return prelude_string_new_dup_fast(string, str, strlen(str));
 }
 
@@ -272,7 +272,7 @@ int prelude_string_new_nodup_fast(prelude_string_t **string, char *str, size_t l
 {
         int ret;
 
-        prelude_return_val_if_fail(str, -1);
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         STRING_RETURN_IF_INVALID(str, len);
 
@@ -303,7 +303,7 @@ int prelude_string_new_nodup_fast(prelude_string_t **string, char *str, size_t l
  */
 int prelude_string_new_nodup(prelude_string_t **string, char *str)
 {
-        prelude_return_val_if_fail(str, -1);
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
         return prelude_string_new_nodup_fast(string, str, strlen(str));
 }
 
@@ -324,7 +324,7 @@ int prelude_string_new_ref_fast(prelude_string_t **string, const char *buf, size
 {
         int ret;
 
-        prelude_return_val_if_fail(buf, -1);
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
         STRING_RETURN_IF_INVALID(buf, len);
 
         ret = prelude_string_new(string);
@@ -352,7 +352,7 @@ int prelude_string_new_ref_fast(prelude_string_t **string, const char *buf, size
  */
 int prelude_string_new_ref(prelude_string_t **string, const char *str)
 {
-        prelude_return_val_if_fail(str, -1);
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
         return prelude_string_new_ref_fast(string, str, strlen(str));
 }
 
@@ -373,7 +373,8 @@ int prelude_string_set_dup_fast(prelude_string_t *string, const char *buf, size_
 {
         int ret;
 
-        prelude_return_val_if_fail(string && buf, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
         STRING_RETURN_IF_INVALID(buf, len);
 
         prelude_string_destroy_internal(string);
@@ -402,7 +403,8 @@ int prelude_string_set_dup_fast(prelude_string_t *string, const char *buf, size_
  */
 int prelude_string_set_dup(prelude_string_t *string, const char *buf)
 {
-        prelude_return_val_if_fail(string && buf, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
         return prelude_string_set_dup_fast(string, buf, strlen(buf));
 }
 
@@ -421,7 +423,8 @@ int prelude_string_set_dup(prelude_string_t *string, const char *buf)
  */
 int prelude_string_set_nodup_fast(prelude_string_t *string, char *buf, size_t len)
 {
-        prelude_return_val_if_fail(string && buf, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
         STRING_RETURN_IF_INVALID(buf, len);
 
         prelude_string_destroy_internal(string);
@@ -450,7 +453,9 @@ int prelude_string_set_nodup_fast(prelude_string_t *string, char *buf, size_t le
  */
 int prelude_string_set_nodup(prelude_string_t *string, char *buf)
 {
-        prelude_return_val_if_fail(string && buf, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
+        
         return prelude_string_set_nodup_fast(string, buf, strlen(buf));
 }
 
@@ -469,7 +474,8 @@ int prelude_string_set_nodup(prelude_string_t *string, char *buf)
  */
 int prelude_string_set_ref_fast(prelude_string_t *string, const char *buf, size_t len)
 {
-        prelude_return_val_if_fail(string && buf, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
         STRING_RETURN_IF_INVALID(buf, len);
 
         prelude_string_destroy_internal(string);
@@ -497,7 +503,9 @@ int prelude_string_set_ref_fast(prelude_string_t *string, const char *buf, size_
  */
 int prelude_string_set_ref(prelude_string_t *string, const char *buf)
 {
-        prelude_return_val_if_fail(string && buf, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
+        
         return prelude_string_set_ref_fast(string, buf, strlen(buf));
 }
 
@@ -516,7 +524,8 @@ int prelude_string_set_ref(prelude_string_t *string, const char *buf)
  */
 int prelude_string_copy_ref(const prelude_string_t *src, prelude_string_t *dst)
 {
-        prelude_return_val_if_fail(src && dst, -1);
+        prelude_return_val_if_fail(src, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(dst, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         prelude_string_destroy_internal(dst);
 
@@ -542,7 +551,9 @@ int prelude_string_copy_ref(const prelude_string_t *src, prelude_string_t *dst)
  */
 int prelude_string_copy_dup(const prelude_string_t *src, prelude_string_t *dst)
 {
-        prelude_return_val_if_fail(src && src->size && dst, -1);
+        prelude_return_val_if_fail(src, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(src->size, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(dst, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         prelude_string_destroy_internal(dst);
 
@@ -575,7 +586,8 @@ int prelude_string_clone(const prelude_string_t *src, prelude_string_t **dst)
 {
         int ret;
 
-        prelude_return_val_if_fail(src && src->size, -1);
+        prelude_return_val_if_fail(src, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(src->size, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         ret = prelude_string_new(dst);
         if ( ret < 0 )
@@ -664,7 +676,7 @@ const char *prelude_string_get_string(const prelude_string_t *string)
  */
 int prelude_string_get_string_released(prelude_string_t *string, char **outptr)
 {
-        prelude_return_val_if_fail(string, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
         *outptr = NULL;
 
         if ( ! string->index )
@@ -774,7 +786,8 @@ int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap
         int ret;
         va_list bkp;
 
-        prelude_return_val_if_fail(string && fmt, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(fmt, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         if ( ! (string->flags & PRELUDE_STRING_CAN_REALLOC) ) {
 
@@ -832,7 +845,8 @@ int prelude_string_sprintf(prelude_string_t *string, const char *fmt, ...)
         int ret;
         va_list ap;
 
-        prelude_return_val_if_fail(string && fmt, -1);
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(fmt, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         va_start(ap, fmt);
         ret = prelude_string_vprintf(string, fmt, ap);
@@ -859,7 +873,8 @@ int prelude_string_ncat(prelude_string_t *dst, const char *str, size_t len)
 {
         int ret;
 
-        prelude_return_val_if_fail(dst && str, -1);
+        prelude_return_val_if_fail(dst, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         if ( dst->flags & PRELUDE_STRING_CAN_REALLOC && len < (dst->size - dst->index) ) {
 
@@ -896,7 +911,9 @@ int prelude_string_ncat(prelude_string_t *dst, const char *str, size_t len)
  */
 int prelude_string_cat(prelude_string_t *dst, const char *str)
 {
-        prelude_return_val_if_fail(dst && str, -1);
+        prelude_return_val_if_fail(dst, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
+        
         return prelude_string_ncat(dst, str, strlen(str));
 }
 
